# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.line import Line
from pygal.view import PolarView
from pygal.util import deg, cached_property
from pygal.interpolate import interpolation
from math import cos, pi


class Radar(Line):
    """Kiviat graph"""

    def _fill(self, values):
        return values

    def _get_value(self, values, i):
        return self.format(values[i][0])

    @cached_property
    def _values(self):
        if self.interpolate:
            return [val[0] for serie in self.series
                    for val in serie.interpolated]
        else:
            return super(Line, self)._values

    def _set_view(self):
        self.view = PolarView(
            self.width - self.margin.x,
            self.height - self.margin.y,
            self._box)

    def _x_axis(self):
        if not self._x_labels:
            return

        axis = self.svg.node(self.plot, class_="axis x web")
        format = lambda x: '%f %f' % x
        center = self.view((0, 0))
        r = self._rmax
        for label, theta in self._x_labels:
            guides = self.svg.node(axis, class_='guides')
            end = self.view((r, theta))
            self.svg.node(guides, 'path',
                      d='M%s L%s' % (format(center), format(end)),
                      class_='line')
            r_txt = (1 - self._box.__class__._margin) * self._box.ymax
            pos_text = self.view((r_txt, theta))
            text = self.svg.node(guides, 'text',
                      x=pos_text[0],
                      y=pos_text[1]
            )
            text.text = label
            angle = - theta + pi / 2.
            if cos(angle) < 0:
                angle -= pi
            text.attrib['transform'] = 'rotate(%f %s)' % (
                deg(angle), format(pos_text))

    def _y_axis(self):
        if not self._y_labels:
            return

        axis = self.svg.node(self.plot, class_="axis y web")

        for label, r in reversed(self._y_labels):
            guides = self.svg.node(axis, class_='guides')
            self.svg.line(
                guides, [self.view((r, theta)) for theta in self._x_pos],
                close=True,
                class_='guide line')
            x, y = self.view((r, self._x_pos[0]))
            self.svg.node(guides, 'text',
                             x=x - 5,
                             y=y
            ).text = label

    def _compute(self):
        delta = 2 * pi / float(self._len)
        self._x_pos = [.5 * pi + i * delta for i in range(self._len + 1)]
        for serie in self.series:
            vals = list(serie.values)
            vals.append(vals[0])
            vals = [val if val != None else 0 for val in vals]
            serie.points = [
                (v, self._x_pos[i])
                for i, v in enumerate(vals)]
            if self.interpolate:
                extend = 2
                extended_x_pos = (
                    [.5 * pi + i * delta for i in range(-extend, 0)] +
                    self._x_pos +
                    [.5 * pi + i * delta for i in range(
                        self._len + 1, self._len + 1 + extend)])
                extended_vals = vals[-extend:] + vals + vals[:extend]
                interpolate = interpolation(
                    extended_x_pos, extended_vals, kind=self.interpolate)
                serie.interpolated = []
                p = float(self.interpolation_precision)
                for s in range(int(p + 1)):
                    x = .5 * pi + 2 * pi * (s / p)
                    serie.interpolated.append((float(interpolate(x)), x))

        self._box._margin *= 2
        self._box.xmin = self._box.ymin = 0
        self._box.xmax = self._box.ymax = self._rmax = max(self._values)

        self._y_pos = self._compute_scale(
            self._box.ymin, self._box.ymax, max_scale=8
        ) if not self.y_labels else map(int, self.y_labels)
        self._x_labels = self.x_labels and zip(self.x_labels, self._x_pos)
        self._y_labels = zip(map(self.format, self._y_pos), self._y_pos)
        self._box.xmin = self._box.ymin = - self._box.ymax



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.graph import Graph
from pygal.util import cached_property
from pygal.interpolate import interpolation
from math import isnan


class Line(Graph):
    """Line graph"""

    def _get_value(self, values, i):
        return self.format(values[i][1])

    @cached_property
    def _values(self):
        if self.interpolate:
            return [val[1]
                    for serie in self.series
                    for val in serie.interpolated
                    if val[1] != None and (not self.logarithmic or val[1] > 0)]
        else:
            return  [val[1]
                     for serie in self.series
                     for val in serie.points
                    if val[1] != None and (not self.logarithmic or val[1] > 0)]

    def _fill(self, values):
        zero = self.view.y(min(max(self.zero, self._box.ymin), self._box.ymax))
        return ([(values[0][0], zero)] +
                values +
                [(values[-1][0], zero)])

    def line(self, serie_node, serie):
        view_values = map(self.view, serie.points)
        if self.show_dots:
            for i, (x, y) in enumerate(view_values):
                if None in (x, y):
                    continue

                classes = []
                if x > self.view.width / 2.:
                    classes.append('left')
                if y > self.view.height / 2.:
                    classes.append('top')
                classes = ' '.join(classes)

                dots = self.svg.node(serie_node['overlay'], class_="dots")
                val = self._get_value(serie.points, i)
                self.svg.node(dots, 'circle', cx=x, cy=y, r=2.5,
                              class_='dot reactive tooltip-trigger')
                self.svg.node(dots, 'desc', class_="value").text = val
                self.svg.node(dots, 'desc',
                              class_="x " + classes).text = str(x)
                self.svg.node(dots, 'desc',
                              class_="y " + classes).text = str(y)
                if self.print_values:
                    self.svg.node(
                        serie_node['text_overlay'], 'text',
                        x=x + self.value_font_size,
                        y=y + self.value_font_size,
                    ).text = val

        if self.stroke:
            if self.interpolate:
                view_values = map(self.view, serie.interpolated)
            if self.fill:
                view_values = self._fill(view_values)
            self.svg.line(
                serie_node['plot'], view_values,
                class_='line reactive' + (' nofill' if not self.fill else ''))

    def _compute(self):
        self._x_pos = [x / float(self._len - 1) for x in range(self._len)
        ] if self._len != 1 else [.5]  # Center if only one value
        for serie in self.series:
            if not hasattr(serie, 'points'):
                serie.points = [
                    (self._x_pos[i], v)
                    for i, v in enumerate(serie.values)]
                if self.interpolate:
                    interpolate = interpolation(
                        self._x_pos, serie.values, kind=self.interpolate)
                    p = float(self.interpolation_precision)
                    serie.interpolated = [
                        (x / p, float(interpolate(x / p)))
                        for x in range(int(p + 1))
                        if not isnan(float(interpolate(x / p)))]
        if self.include_x_axis:
            self._box.ymin = min(min(self._values), 0)
            self._box.ymax = max(max(self._values), 0)
        else:
            self._box.ymin = min(self._values)
            self._box.ymax = max(self._values)

        self._y_pos = self._compute_scale(self._box.ymin, self._box.ymax
        ) if not self.y_labels else map(float, self.y_labels)

        self._x_labels = self.x_labels and zip(self.x_labels, self._x_pos)
        self._y_labels = zip(map(self.format, self._y_pos), self._y_pos)

    def _plot(self):
        for serie in self.series:
            self.line(self._serie(serie.index), serie)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from math import sin, cos, log10


class Margin(object):
    def __init__(self, top, right, bottom, left):
        self.top = top
        self.right = right
        self.bottom = bottom
        self.left = left

    @property
    def x(self):
        return self.left + self.right

    @property
    def y(self):
        return self.top + self.bottom


class Box(object):
    _margin = .02

    def __init__(self):
        self.xmin = self.ymin = 0
        self.xmax = self.ymax = 1

    @property
    def width(self):
        return self.xmax - self.xmin

    @property
    def height(self):
        return self.ymax - self.ymin

    def swap(self):
        self.xmin, self.ymin = self.ymin, self.xmin
        self.xmax, self.ymax = self.ymax, self.xmax

    def fix(self, with_margin=True):
        if not self.width:
            self.xmax = self.xmin + 1
        if not self.height:
            self.ymin -= .5
            self.ymax = self.ymin + 1
        xmargin = self._margin * self.width
        self.xmin -= xmargin
        self.xmax += xmargin
        if with_margin:
            ymargin = self._margin * self.height
            self.ymin -= ymargin
            self.ymax += ymargin


class View(object):
    def __init__(self, width, height, box):
        self.width = width
        self.height = height
        self.box = box
        self.box.fix()

    def x(self, x):
        if x == None:
            return None
        return self.width * (x - self.box.xmin) / float(self.box.width)

    def y(self, y):
        if y == None:
            return None
        return (self.height - self.height *
                (y - self.box.ymin) / float(self.box.height))

    def __call__(self, xy):
        x, y = xy
        return (self.x(x), self.y(y))


class PolarView(View):
    def __call__(self, rhotheta):
        if None in rhotheta:
            return None, None
        rho, theta = rhotheta
        rho = max(rho, 0)
        return super(PolarView, self).__call__(
            (rho * cos(theta), rho * sin(theta)))


class LogView(View):
    def __init__(self, width, height, box):
        self.width = width
        self.height = height
        self.box = box
        self.ymin = self.box.ymin
        self.ymax = self.box.ymax
        self.log10_ymax = log10(self.box.ymax)
        self.log10_ymin = log10(self.box.ymin)
        self.box.fix(False)

    def y(self, y):
        if y == None or y <= 0:
            return None
        return (self.height - self.height *
                (log10(y) - self.log10_ymin)
                / float(self.log10_ymax - self.log10_ymin))



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.

__version__ = '0.9.20'

from pygal.graph.bar import Bar
from pygal.graph.horizontal import HorizontalBar
from pygal.graph.stackedbar import StackedBar
from pygal.graph.horizontal import HorizontalStackedBar
from pygal.graph.line import Line
from pygal.graph.stackedline import StackedLine
from pygal.graph.xy import XY
from pygal.graph.pie import Pie
from pygal.graph.radar import Radar
from pygal.config import Config

"""List of all chart types"""
CHARTS = [
    Bar, HorizontalBar, StackedBar, HorizontalStackedBar,
    Line, StackedLine, XY, Pie, Radar]



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from decimal import Decimal
from math import floor, pi, log, log10
ORDERS = u"yzafpnµm kMGTPEZY"


def float_format(number):
    return ("%.3f" % number).rstrip('0').rstrip('.')


def humanize(number):
    order = number and int(floor(log(abs(number)) / log(1000)))
    human_readable = ORDERS.split(" ")[int(order > 0)]
    if order == 0 or order > len(human_readable):
        return float_format(number / float(1000 ** int(order)))
    return (
        float_format(number / float(1000 ** int(order))) +
        human_readable[int(order) - int(order > 0)])


def is_major(number):
    """Returns True if number is a round order: 1, 100, 0.001"""
    return not number or 10 ** floor(log10(abs(number))) == abs(number)


def round_to_int(number, precision):
    precision = int(precision)
    rounded = (int(number) + precision / 2) / precision * precision
    return rounded


def round_to_float(number, precision):
    rounded = Decimal(
        floor((number + precision / 2) / precision)) * Decimal(str(precision))
    return float(rounded)


def round_to_scale(number, precision):
    if precision < 1:
        return round_to_float(number, precision)
    return round_to_int(number, precision)


def cut(list_, index=0):
    if isinstance(index, int):
        cut = lambda x: x[index]
    else:
        cut = lambda x: getattr(x, index)
    return map(cut, list_)


def rad(deg):
    return pi * deg / 180.


def deg(deg):
    return 180 * deg / pi


def _swap_curly(string):
    """Swap single and double curly brackets"""
    return (string
            .replace('{{ ', '{{')
            .replace('{{', '\x00')
            .replace('{', '{{')
            .replace('\x00', '{')
            .replace(' }}', '}}')
            .replace('}}', '\x00')
            .replace('}', '}}')
            .replace('\x00', '}'))


def template(string, **kwargs):
    """Format a string using double braces"""
    return _swap_curly(string).format(**kwargs)

swap = lambda tuple_: tuple(reversed(tuple_))
ident = lambda x: x


# Stolen from brownie http://packages.python.org/Brownie/
class cached_property(object):
    def __init__(self, getter, doc=None):
        self.getter = getter
        self.__module__ = getter.__module__
        self.__name__ = getter.__name__
        self.__doc__ = doc or getter.__doc__

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        value = obj.__dict__[self.__name__] = self.getter(obj)
        return value



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
import os
from pygal import Line
import pygal


def test_multi_render():
    for Chart in pygal.CHARTS:
        chart = Chart()
        rng = range(20)
        if Chart == pygal.XY:
            rng = zip(rng, rng)
        chart.add('Serie', rng)
        chart.add('Serie 2', list(reversed(rng)))
        svg = chart.render()
        for i in range(2):
            assert svg == chart.render()


def test_render_to_file():
    file_name = '/tmp/test_graph.svg'
    if os.path.exists(file_name):
        os.remove(file_name)

    line = Line()
    line.add('Serie 1', [1])
    line.render_to_file(file_name)
    with open(file_name) as f:
        assert 'pygal' in f.read()
    os.remove(file_name)


def test_render_to_png():
    file_name = '/tmp/test_graph.png'
    if os.path.exists(file_name):
        os.remove(file_name)

    line = Line()
    line.add('Serie 1', [1])
    line.render_to_png(file_name)
    with open(file_name) as f:
        assert f.read()
    os.remove(file_name)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.util import ident
try:
    import scipy
    from scipy import interpolate
except:
    scipy = None


def interpolation(x, y, kind):
    assert scipy != None, 'You must have scipy installed to use interpolation'
    order = None
    if len(y) < len(x):
        x = x[:len(y)]

    x, y = zip(*filter(lambda t: None not in t, zip(x, y)))

    if len(x) < 2:
        return ident
    if isinstance(kind, int):
        order = kind
    elif kind in ['zero', 'slinear', 'quadratic', 'cubic', 'univariate']:
        order = {'nearest': 0, 'zero': 0, 'slinear': 1,
                 'quadratic': 2, 'cubic': 3, 'univariate': 3}[kind]
    if order and len(x) <= order:
        kind = len(x) - 1
    if kind == 'krogh':
        return interpolate.KroghInterpolator(x, y)
    elif kind == 'barycentric':
        return interpolate.BarycentricInterpolator(x, y)
    elif kind == 'univariate':
        return interpolate.InterpolatedUnivariateSpline(x, y)
    return interpolate.interp1d(x, y, kind=kind, bounds_error=False)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.line import Line
from pygal.interpolate import interpolation
from math import isnan


class XY(Line):
    """XY Line graph"""

    def _get_value(self, values, i):
        return 'x=%s, y=%s' % tuple(map(self.format, values[i]))

    def _compute(self):
        xvals = [val[0]
                 for serie in self.series
                 for val in serie.values
                 if val[0] != None]
        yvals = [val[1]
                 for serie in self.series
                 for val in serie.values
                 if val[1] != None]
        xmin = min(xvals)

        for serie in self.series:
            serie.points = serie.values
            if self.interpolate:
                vals = zip(*serie.points)
                interpolate = interpolation(
                    vals[0], vals[1], kind=self.interpolate)
                serie_xmin = min(vals[0])
                serie_xmax = max(vals[0])
                serie.interpolated = []
                r = (max(xvals) - xmin)
                p = float(self.interpolation_precision)
                for s in range(int(p + 1)):
                    x = xmin + r * (s / p)
                    if (serie_xmin <= x <= serie_xmax and not
                        isnan(float(interpolate(x)))):
                        serie.interpolated.append((x, float(interpolate(x))))

        if self.interpolate:
            xvals = [val[0]
                     for serie in self.series
                     for val in serie.interpolated]
            yvals = [val[1]
                     for serie in self.series
                     for val in serie.interpolated]

        self._box.xmin, self._box.xmax = min(xvals), max(xvals)
        self._box.ymin, self._box.ymax = min(yvals), max(yvals)
        x_pos = self._compute_scale(self._box.xmin, self._box.xmax)
        y_pos = self._compute_scale(self._box.ymin, self._box.ymax)

        self._x_labels = zip(map(self.format, x_pos), x_pos)
        self._y_labels = zip(map(self.format, y_pos), y_pos)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.util import (
    round_to_int, round_to_float, _swap_curly, template, humanize,
    is_major)
from pytest import raises


def test_round_to_int():
    assert round_to_int(154231, 1000) == 154000
    assert round_to_int(154231, 10) == 154230
    assert round_to_int(154231, 100000) == 200000
    assert round_to_int(154231, 50000) == 150000
    assert round_to_int(154231, 500) == 154000
    assert round_to_int(154231, 200) == 154200
    assert round_to_int(154361, 200) == 154400


def test_round_to_float():
    assert round_to_float(12.01934, .01) == 12.02
    assert round_to_float(12.01134, .01) == 12.01
    assert round_to_float(12.1934, .1) == 12.2
    assert round_to_float(12.1134, .1) == 12.1
    assert round_to_float(12.1134, .001) == 12.113
    assert round_to_float(12.1134, .00001) == 12.1134
    assert round_to_float(12.1934, .5) == 12.0
    assert round_to_float(12.2934, .5) == 12.5


def test_swap_curly():
    for str in (
            'foo',
            u'foo foo foo bar',
            'foo béè b¡ð/ĳə˘©þß®~¯æ',
            u'foo béè b¡ð/ĳə˘©þß®~¯æ'):
        assert _swap_curly(str) == str
    assert _swap_curly('foo{bar}baz') == 'foo{{bar}}baz'
    assert _swap_curly('foo{{bar}}baz') == 'foo{bar}baz'
    assert _swap_curly('{foo}{{bar}}{baz}') == '{{foo}}{bar}{{baz}}'
    assert _swap_curly('{foo}{{{bar}}}{baz}') == '{{foo}}{{{bar}}}{{baz}}'
    assert _swap_curly('foo{ bar }baz') == 'foo{{ bar }}baz'
    assert _swap_curly('foo{ bar}baz') == 'foo{{ bar}}baz'
    assert _swap_curly('foo{bar }baz') == 'foo{{bar }}baz'
    assert _swap_curly('foo{{ bar }}baz') == 'foo{bar}baz'
    assert _swap_curly('foo{{bar }}baz') == 'foo{bar}baz'
    assert _swap_curly('foo{{ bar}}baz') == 'foo{bar}baz'


def test_format():
    assert template('foo {{ baz }}', baz='bar') == 'foo bar'
    with raises(KeyError):
        assert template('foo {{ baz }}') == 'foo baz'

    class Object(object):
        pass
    obj = Object()
    obj.a = 1
    obj.b = True
    obj.c = '3'
    assert template('foo {{ o.a }} {{o.b}}-{{o.c}}',
               o=obj) == 'foo 1 True-3'


def test_humanize():
    assert humanize(1) == '1'
    assert humanize(1.) == '1'
    assert humanize(10) == '10'
    assert humanize(12.5) == '12.5'
    assert humanize(1000) == '1k'
    assert humanize(5000) == '5k'
    assert humanize(100000) == '100k'
    assert humanize(1253) == '1.253k'
    assert humanize(1250) == '1.25k'

    assert humanize(0.1) == '100m'
    assert humanize(0.01) == '10m'
    assert humanize(0.001) == '1m'
    assert humanize(0.002) == '2m'
    assert humanize(0.0025) == '2.5m'
    assert humanize(0.0001) == u'100µ'
    assert humanize(0.000123) == u'123µ'
    assert humanize(0.00001) == u'10µ'
    assert humanize(0.000001) == u'1µ'
    assert humanize(0.0000001) == u'100n'
    assert humanize(0.0000000001) == u'100p'

    assert humanize(0) == '0'
    assert humanize(0.) == '0'
    assert humanize(-1337) == '-1.337k'
    assert humanize(-.000000042) == '-42n'


def test_is_major():
    for n in (0, 1, 1000, 10., 0.1, 0.000001, -10, -.001000, -100.):
        assert is_major(n)
    for n in (2, 10002., 100000.0003, -200, -0.0005):
        assert not is_major(n)



from pygal.serie import Serie
from pygal.view import Margin, Box
from pygal.util import round_to_scale, cut, rad, humanize
from pygal.svg import Svg
from pygal.config import Config
from pygal.util import cached_property
from math import log10, sin, cos, floor, ceil


class BaseGraph(object):
    """Graphs commons"""

    def __init__(self, config=None, **kwargs):
        self.config = config or Config()
        self.config(**kwargs)
        self.svg = Svg(self)
        self.series = []
        self._x_labels = self._y_labels = None

    def _init(self):
        self.margin = Margin(*([20] * 4))
        self._box = Box()

    def __getattr__(self, attr):
        if attr in dir(self.config):
            return object.__getattribute__(self.config, attr)
        return object.__getattribute__(self, attr)

    @property
    def format(self):
        return humanize if self.human_readable else str

    def _compute_logarithmic_scale(self, min_, max_):
        min_order = int(floor(log10(min_)))
        max_order = int(ceil(log10(max_)))
        positions = []
        amplitude = max_order - min_order
        if amplitude <= 1:
            return []
        detail = 10.
        while amplitude * detail < 20:
            detail *= 2
        while amplitude * detail > 50:
            detail /= 2
        for order in range(min_order, max_order + 1):
            for i in range(int(detail)):
                tick = (10 * i / detail or 1.) * 10 ** order
                tick = round_to_scale(tick, tick)
                if min_ <= tick <= max_ and tick not in positions:
                    positions.append(tick)
        return positions

    def _compute_scale(self, min_, max_, min_scale=4, max_scale=20):
        if min_ == 0 and max_ == 0:
            return [0]
        if max_ - min_ == 0:
            return [min_]
        if self.logarithmic:
            log_scale = self._compute_logarithmic_scale(min_, max_)
            if log_scale:
                return log_scale
                # else we fallback to normal scalling
        order = round(log10(max(abs(min_), abs(max_)))) - 1
        while (max_ - min_) / float(10 ** order) < min_scale:
            order -= 1
        step = float(10 ** order)
        while (max_ - min_) / step > max_scale:
            step *= 2.
        positions = []
        position = round_to_scale(min_, step)
        while position < (max_ + step):
            rounded = round_to_scale(position, step)
            if min_ <= rounded <= max_:
                if rounded not in positions:
                    positions.append(rounded)
            position += step
        if len(positions) < 2:
            return [min_, max_]
        return positions

    def _text_len(self, lenght, fs):
        return lenght * 0.6 * fs

    def _get_text_box(self, text, fs):
        return (fs, self._text_len(len(text), fs))

    def _get_texts_box(self, texts, fs):
        max_len = max(map(len, texts))
        return (fs, self._text_len(max_len, fs))

    def _compute(self):
        """Initial computations to draw the graph"""

    def _compute_margin(self):
        if self.show_legend:
            h, w = self._get_texts_box(
                cut(self.series, 'title'), self.legend_font_size)
            self.margin.right += 10 + w + self.legend_box_size

        if self.title:
            h, w = self._get_text_box(self.title, self.title_font_size)
            self.margin.top += 10 + h

        if self._x_labels:
            h, w = self._get_texts_box(
                cut(self._x_labels), self.label_font_size)
            self.margin.bottom += 10 + max(
                w * sin(rad(self.x_label_rotation)), h)
            if self.x_label_rotation:
                self.margin.right = max(
                    w * cos(rad(self.x_label_rotation)),
                    self.margin.right)
        if self._y_labels:
            h, w = self._get_texts_box(
                cut(self._y_labels), self.label_font_size)
            self.margin.left += 10 + max(
                w * cos(rad(self.y_label_rotation)), h)

    @cached_property
    def _legends(self):
        return [serie.title for serie in self.series]

    @cached_property
    def _values(self):
        return [val
                for serie in self.series
                for val in serie.values
                if val != None]

    @cached_property
    def _len(self):
        return max([len(serie.values) for serie in self.series])

    def _draw(self):
        self._compute()
        self._compute_margin()
        self._decorate()
        self._plot()

    def add(self, title, values):
        self.series.append(Serie(title, values, len(self.series)))

    def _has_data(self):
        if len(self.series) == 0:
            return False
        for serie in self.series:
            if not hasattr(serie.values, '__iter__'):
                serie.values = [serie.values]
        if sum(map(len, map(lambda s: s.values, self.series))) == 0:
            return False
        return True

    def _render(self):
        self._init()
        self.svg._init()
        if self._has_data():
            self._draw()
            self.svg._pre_render(False)
        else:
            self.svg._pre_render(True)

    def render(self):
        self._render()
        return self.svg.render()

    def render_tree(self):
        self._render()
        return self.svg.root

    def render_pyquery(self):
        from pyquery import PyQuery as pq
        return pq(self.render_tree())

    def render_in_browser(self):
        from lxml.html import open_in_browser
        open_in_browser(self.render_tree(), encoding='utf-8')

    def render_response(self):
        from flask import Response
        return Response(self.render(), mimetype='image/svg+xml')

    def render_to_file(self, filename):
        with open(filename, 'w') as f:
            f.write(self.render())

    def render_to_png(self, filename):
        import cairosvg
        import StringIO
        fakefile = StringIO.StringIO()
        fakefile.write(self.render())
        fakefile.seek(0)
        cairosvg.surface.PNGSurface.convert(
            file_obj=fakefile, write_to=filename)



from pygal.graph.base import BaseGraph
from pygal.view import View, LogView
from pygal.util import is_major


class Graph(BaseGraph):
    """Graph super class containing generic common functions"""

    def _decorate(self):
        self._set_view()
        self._make_graph()
        self._x_axis()
        self._y_axis()
        self._legend()
        self._title()

    def _set_view(self):
        self.view = (LogView if self.logarithmic else View)(
            self.width - self.margin.x,
            self.height - self.margin.y,
            self._box)

    def _make_graph(self):
        self.graph_node = self.svg.node(
            class_='graph %s-graph' % self.__class__.__name__.lower())
        self.svg.node(self.graph_node, 'rect',
                  class_='background',
                  x=0, y=0,
                  width=self.width,
                  height=self.height)
        self.plot = self.svg.node(
            self.graph_node, class_="plot",
            transform="translate(%d, %d)" % (
                self.margin.left, self.margin.top))
        self.svg.node(self.plot, 'rect',
                  class_='background',
                  x=0, y=0,
                  width=self.view.width,
                  height=self.view.height)
        self.overlay = self.svg.node(
            self.graph_node, class_="plot overlay",
            transform="translate(%d, %d)" % (
                self.margin.left, self.margin.top))
        self.text_overlay = self.svg.node(
            self.graph_node, class_="plot text-overlay",
            transform="translate(%d, %d)" % (
                self.margin.left, self.margin.top))
        tooltip_overlay = self.svg.node(
            self.graph_node, class_="plot tooltip-overlay",
            transform="translate(%d, %d)" % (
                self.margin.left, self.margin.top))
        self.tooltip_node = self.svg.node(tooltip_overlay, id="tooltip",
                                          transform='translate(0 0)')

        self.svg.node(self.tooltip_node, 'rect',
                      id="tooltip-box",
                      rx=5, ry=5,
        )
        self.svg.node(self.tooltip_node, 'text')

    def _x_axis(self):
        if not self._x_labels:
            return

        axis = self.svg.node(self.plot, class_="axis x")

        if 0 not in [label[1] for label in self._x_labels]:
            self.svg.node(axis, 'path',
                      d='M%f %f v%f' % (0, 0, self.view.height),
                      class_='line')
        for label, position in self._x_labels:
            guides = self.svg.node(axis, class_='guides')
            x = self.view.x(position)
            y = self.view.height + 5
            self.svg.node(guides, 'path',
                      d='M%f %f v%f' % (x, 0, self.view.height),
                    class_='%sline' % (
                        'guide ' if position != 0 else ''))
            text = self.svg.node(guides, 'text',
                             x=x,
                             y=y + .5 * self.label_font_size + 5
            )
            text.text = label
            if self.x_label_rotation:
                text.attrib['transform'] = "rotate(%d %f %f)" % (
                    self.x_label_rotation, x, y)

    def _y_axis(self):
        if not self._y_labels:
            return

        axis = self.svg.node(self.plot, class_="axis y")

        if 0 not in [label[1] for label in self._y_labels]:
            self.svg.node(axis, 'path',
                      d='M%f %f h%f' % (0, self.view.height, self.view.width),
                      class_='line')
        for label, position in self._y_labels:
            major = is_major(position)
            guides = self.svg.node(axis, class_='%sguides' % (
                'logarithmic ' if self.logarithmic else ''
            ))
            x = -5
            y = self.view.y(position)
            self.svg.node(guides, 'path',
                          d='M%f %f h%f' % (0, y, self.view.width),
                          class_='%s%sline' % (
                              'major ' if major else '',
                              'guide ' if position != 0 else ''))
            text = self.svg.node(guides, 'text',
                                 x=x,
                                 y=y + .35 * self.label_font_size,
                                 class_='major' if major else ''
            )
            text.text = label
            if self.y_label_rotation:
                text.attrib['transform'] = "rotate(%d %f %f)" % (
                    self.y_label_rotation, x, y)

    def _legend(self):
        if not self.show_legend:
            return
        legends = self.svg.node(
            self.graph_node, class_='legends',
            transform='translate(%d, %d)' % (
                self.margin.left + self.view.width + 10,
                self.margin.top + 10))
        for i, title in enumerate(self._legends):
            legend = self.svg.node(
                legends, class_='legend reactive activate-serie',
                id="activate-serie-%d" % i)
            self.svg.node(
                legend, 'rect',
                x=0,
                y=1.5 * i * self.legend_box_size,
                width=self.legend_box_size,
                height=self.legend_box_size,
                class_="color-%d reactive" % i
            ).text = title
            # Serious magical numbers here
            self.svg.node(
                legend, 'text',
                x=self.legend_box_size + 5,
                y=1.5 * i * self.legend_box_size
                + .5 * self.legend_box_size
                + .3 * self.legend_font_size
            ).text = title

    def _title(self):
        if self.title:
            self.svg.node(self.graph_node, 'text', class_='title',
                      x=self.margin.left + self.view.width / 2,
                      y=self.title_font_size + 10
            ).text = self.title

    def _serie(self, serie):
        return dict(
            plot=self.svg.node(
                self.plot,
                class_='series serie-%d color-%d' % (serie, serie)),
            overlay=self.svg.node(
                self.overlay,
                class_='series serie-%d color-%d' % (serie, serie)),
            text_overlay=self.svg.node(
                self.text_overlay,
                class_='series serie-%d color-%d' % (serie, serie)))



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal import Line
from pygal.test.utils import texts
from math import cos, sin


def test_simple_line():
    line = Line()
    rng = range(-30, 31, 5)
    line.add('test1', [cos(x / 10.) for x in rng])
    line.add('test2', [sin(x / 10.) for x in rng])
    line.add('test3', [cos(x / 10.) - sin(x / 10.) for x in rng])
    line.x_labels = map(str, rng)
    line.title = "cos sin and cos - sin"
    q = line.render_pyquery()
    assert len(q(".axis.x")) == 1
    assert len(q(".axis.y")) == 1
    assert len(q(".plot .series path")) == 3
    assert len(q(".legend")) == 3
    assert len(q(".x.axis .guides")) == 13
    assert len(q(".y.axis .guides")) == 13
    assert len(q(".dots")) == 3 * 13
    assert q(".axis.x text").map(texts) == [
        '-30', '-25', '-20', '-15', '-10', '-5',
        '0', '5', '10', '15', '20', '25', '30']
    assert q(".axis.y text").map(texts) == [
        '-1.2', '-1.0', '-0.8', '-0.6', '-0.4', '-0.2',
        '0.0', '0.2', '0.4', '0.6', '0.8', '1.0', '1.2']
    assert q(".title").text() == 'cos sin and cos - sin'
    assert q(".legend text").map(texts) == ['test1', 'test2', 'test3']


def test_line():
    line = Line()
    rng = [8, 12, 23, 73, 39, 57]
    line.add('Single serie', rng)
    line.title = "One serie"
    q = line.render_pyquery()
    assert len(q(".axis.x")) == 0
    assert len(q(".axis.y")) == 1
    assert len(q(".plot .series path")) == 1
    assert len(q(".x.axis .guides")) == 0
    assert len(q(".y.axis .guides")) == 7


def test_one_dot():
    line = Line()
    line.add('one dot', [12])
    line.x_labels = ['one']
    q = line.render_pyquery()
    assert len(q(".axis.x")) == 1
    assert len(q(".axis.y")) == 1
    assert len(q(".y.axis .guides")) == 1


def test_no_dot():
    line = Line()
    line.add('no dot', [])
    q = line.render_pyquery()
    assert q(".plot") == []
    assert q("text").text() == 'No data'


def test_no_dot_at_all():
    q = Line().render_pyquery()
    assert q(".plot") == []
    assert q("text").text() == 'No data'



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.graph import Graph
from pygal.util import swap, ident


class Bar(Graph):
    """Bar graph"""

    def bar(self, serie_node, serie, values, stack_vals=None):
        """Draw a bar graph for a serie"""
        # value here is a list of tuple range of tuple coord

        def view(rng):
            """Project range"""
            t, T = rng
            fun = swap if self._horizontal else ident
            return self.view(fun(t)), self.view(fun(T))

        bars = self.svg.node(serie_node['plot'], class_="bars")
        view_values = map(view, values)
        for i, ((x, y), (X, Y)) in enumerate(view_values):
            if None in (x, y):
                continue
            # x and y are left range coords and X, Y right ones
            val = self.format(values[i][1][1])
            if self._horizontal:
                x, y, X, Y = Y, X, y, x
            width = X - x
            padding = .1 * width
            inner_width = width - 2 * padding
            if self._horizontal:
                height = self.view.x(0) - y
            else:
                height = self.view.y(0) - y
            if stack_vals == None:
                bar_width = inner_width / len(self.series)
                bar_padding = .1 * bar_width
                bar_inner_width = bar_width - 2 * bar_padding
                offset = serie.index * bar_width + bar_padding
                shift = 0
            else:
                offset = 0
                bar_inner_width = inner_width
                shift = stack_vals[i][int(height < 0)]
                stack_vals[i][int(height < 0)] += height
            x = x + padding + offset

            if height < 0:
                y = y + height
                height = -height
            y -= shift
            bar = self.svg.node(bars, class_='bar')
            self.svg.transposable_node(
                bar, 'rect',
                x=x,
                y=y,
                rx=self.rounded_bars * 1,
                ry=self.rounded_bars * 1,
                width=bar_inner_width,
                height=height,
                class_='rect reactive tooltip-trigger')
            self.svg.node(bar, 'desc', class_="values").text = val
            tooltip_positions = map(
                str, (x + bar_inner_width / 2., y + height / 2.))
            self.svg.node(bar, 'desc',
                          class_="x centered"
            ).text = tooltip_positions[int(self._horizontal)]
            self.svg.node(bar, 'desc',
                          class_="y centered"
            ).text = tooltip_positions[int(not self._horizontal)]
            if self._horizontal:
                x += .3 * self.value_font_size
                y += height / 2
            else:
                y += height / 2 + .3 * self.value_font_size
            if self.print_values:
                self.svg.transposable_node(
                    serie_node['text_overlay'], 'text',
                    class_='centered',
                    x=x + bar_inner_width / 2,
                    y=y
                ).text = val if self.print_zeroes or val != '0' else ''
        return stack_vals

    def _compute(self):
        self._box.ymin = min(min(self._values), self.zero)
        self._box.ymax = max(max(self._values), self.zero)
        x_step = len(self.series[0].values)
        x_pos = [x / float(x_step) for x in range(x_step + 1)
        ] if x_step > 1 else [0, 1]  # Center if only one value
        y_pos = self._compute_scale(self._box.ymin, self._box.ymax,
        ) if not self.y_labels else map(float, self.y_labels)

        self._x_ranges = zip(x_pos, x_pos[1:])
        self._x_labels = self.x_labels and zip(self.x_labels, [
            sum(x_range) / 2. for x_range in self._x_ranges])
        self._y_labels = zip(map(self.format, y_pos), y_pos)

    def _plot(self):
        for serie in self.series:
            serie_node = self._serie(serie.index)
            self.bar(serie_node, serie, [
                tuple((self._x_ranges[i][j], v) for j in range(2))
                for i, v in enumerate(serie.values)])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
import argparse
import pygal

parser = argparse.ArgumentParser(
    description='Generate pygal chart in command line',
    prog='pygal_gen',
    version=pygal.__version__)

parser.add_argument('-t', '--type', dest='type', default='Line',
                    choices=map(lambda x: x.__name__, pygal.CHARTS),
                    help='Kind of chart to generate')

parser.add_argument('-o', '--output', dest='filename', default='pygal_out.svg',
                    help='Filename to write the svg to')

parser.add_argument('-s', '--serie', dest='series', nargs='+', action='append',
                    help='Add a serie in the form (title val1 val2...)')

for key, val in pygal.config.Config.__dict__.items():
    if not key.startswith('_') and not hasattr(val, '__call__'):
        opt_name = key
        opts = {'type': str}
        if val != None:
            opts['type'] = type(val)
        elif 'labels' in key:
            opts['nargs'] = '+'
        if opts['type'] == bool:
            del opts['type']
            opts['action'] = 'store_true' if not val else 'store_false'
            if val:
                opt_name = 'no-' + opt_name
        if key == 'interpolate':
            opts['choices'] = [
                'linear', 'nearest', 'zero', 'slinear', 'quadratic',
                'cubic', 'krogh', 'barycentric', 'univariate']
        parser.add_argument(
            '--%s' % opt_name, dest=key, default=val, **opts)

config = parser.parse_args()

chart = getattr(pygal, config.type)(**vars(config))

for serie in config.series:
    chart.add(serie[0], map(float, serie[1:]))

chart.render_to_file(config.filename)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.


class Style(object):
    def __init__(self,
        background='black',
        plot_background='#111',
        foreground='#999',
        foreground_light='#eee',
        foreground_dark='#555',
        opacity='.8',
        opacity_hover='.9',
        transition='250ms',
        colors=(
                '#ff5995', '#b6e354', '#feed6c', '#8cedff', '#9e6ffe',
                '#899ca1', '#f8f8f2', '#808384', '#bf4646', '#516083',
                '#f92672', '#82b414', '#fd971f', '#56c2d6', '#8c54fe',
                '#465457')):
            self.background = background
            self.plot_background = plot_background
            self.foreground = foreground
            self.foreground_light = foreground_light
            self.foreground_dark = foreground_dark
            self.opacity = opacity
            self.opacity_hover = opacity_hover
            self.transition = transition
            self._colors = colors

    @property
    def colors(self):
        def color(tupl):
            return (
                    '.color-{0} {{\n'
                    '  stroke: {1};\n'
                    '  fill: {1};\n'
                    '}}\n'.format(*tupl))
        return '\n'.join(map(color, enumerate(self._colors)))

DefaultStyle = Style(opacity_hover='.4', opacity='.8')
LightStyle = Style(
    background='white',
    plot_background='rgba(0, 0, 255, 0.1)',
    foreground='rgba(0, 0, 0, 0.7)',
    foreground_light='rgba(0, 0, 0, 0.9)',
    foreground_dark='rgba(0, 0, 0, 0.5)',
    colors=('#242424', '#9f6767', '#92ac68',
            '#d0d293', '#9aacc3', '#bb77a4',
            '#77bbb5', '#777777'))
NeonStyle = Style(
    opacity='.1',
    opacity_hover='.75',
    transition='1s ease-out')
CleanStyle = Style(
    background='transparent',
    plot_background='rgba(240, 240, 240, 0.7)',
    foreground='rgba(0, 0, 0, 0.9)',
    foreground_light='rgba(0, 0, 0, 0.9)',
    foreground_dark='rgba(0, 0, 0, 0.5)',
    colors=(
        'rgb(12,55,149)', 'rgb(117,38,65)', 'rgb(228,127,0)', 'rgb(159,170,0)',
        'rgb(149,12,12)'))

solarized_colors = (
    '#b58900', '#cb4b16', '#dc322f', '#d33682',
    '#6c71c4', '#268bd2', '#2aa198', '#859900')

DarkSolarizedStyle = Style(
    background='#073642',
    plot_background='#002b36',
    foreground='#839496',
    foreground_light='#fdf6e3',
    foreground_dark='#657b83',
    opacity='.66',
    opacity_hover='.9',
    transition='500ms ease-in',
    colors=solarized_colors)

LightSolarizedStyle = Style(
    background='#fdf6e3',
    plot_background='#eee8d5',
    foreground='#657b83',
    foreground_light='#073642',
    foreground_dark='#073642',
    opacity='.6',
    opacity_hover='.9',
    transition='500ms ease-in',
    colors=solarized_colors)


styles = {'default': DefaultStyle,
          'light': LightStyle,
          'neon': NeonStyle,
          'clean': CleanStyle,
          'dark_solarized': DarkSolarizedStyle,
          'light_solarized': LightSolarizedStyle}



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.graph import Graph
from math import cos, sin, pi
project = lambda rho, alpha: (
    rho * sin(-alpha), rho * cos(-alpha))
diff = lambda x, y: (x[0] - y[0], x[1] - y[1])
fmt = lambda x: '%f %f' % x
get_radius = lambda r: fmt(tuple([r] * 2))


class Pie(Graph):
    """Pie graph"""

    def slice(self, serie_node, start_angle, angle, perc,
            small=False):
        val = '{:.2%}'.format(perc)
        slices = self.svg.node(serie_node['plot'], class_="slices")
        slice_ = self.svg.node(slices, class_="slice")
        center = ((self.width - self.margin.x) / 2.,
                  (self.height - self.margin.y) / 2.)
        r = min(center)
        if small:
            small_r = r * .9
        else:
            r = r * .9
            small_r = 0
        if perc == 1:
            self.svg.node(slice_, 'circle',
                          cx=center[0],
                          cy=center[1],
                          r=r,
                          class_='slice reactive tooltip-trigger')
        else:
            absolute_project = lambda rho, theta: fmt(
                diff(center, project(rho, theta)))
            to1 = absolute_project(r, start_angle)
            to2 = absolute_project(r, start_angle + angle)
            to3 = absolute_project(small_r, start_angle + angle)
            to4 = absolute_project(small_r, start_angle)
            self.svg.node(slice_, 'path',
                          d='M%s A%s 0 %d 1 %s L%s A%s 0 %d 0 %s z' % (
                              to1,
                              get_radius(r), int(angle > pi), to2,
                              to3,
                              get_radius(small_r), int(angle > pi), to4),
                          class_='slice reactive tooltip-trigger')
        self.svg.node(slice_, 'desc', class_="value").text = val
        tooltip_position = map(
            str, diff(center, project(
                (r + small_r) / 2., start_angle + angle / 2.)))
        self.svg.node(slice_, 'desc',
                      class_="x centered").text = tooltip_position[0]
        self.svg.node(slice_, 'desc',
                      class_="y centered").text = tooltip_position[1]
        if self.print_values:
            self.svg.node(
                serie_node['text_overlay'], 'text',
                class_='centered',
                x=tooltip_position[0],
                y=tooltip_position[1]
            ).text = val if self.print_zeroes or val != '0%' else ''

    def _compute(self):
        for serie in self.series:
            serie.values = map(lambda x: max(x, 0), serie.values)
        return super(Pie, self)._compute()

    def _plot(self):
        total = float(sum(map(sum, map(lambda x: x.values, self.series))))

        if total == 0:
            return
        current_angle = 0
        for serie in self.series:
            angle = 2 * pi * sum(serie.values) / total
            self.slice(
                self._serie(serie.index),
                current_angle,
                angle, sum(serie.values) / total)
            if len(serie.values) > 1:
                small_current_angle = current_angle
                for i, val in enumerate(serie.values):
                    small_angle = 2 * pi * val / total
                    self.slice(
                        self._serie(serie.index),
                        small_current_angle,
                        small_angle, val / total,
                        True)
                    small_current_angle += small_angle
            current_angle += angle



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.line import Line
from pygal.interpolate import interpolation
from math import isnan
from itertools import izip_longest


class StackedLine(Line):
    """Stacked Line graph"""

    def _fill(self, values):
        if not hasattr(self, '_previous_line'):
            self._previous_line = values
            return super(StackedLine, self)._fill(values)
        new_values = values + list(reversed(self._previous_line))
        self._previous_line = values
        return new_values

    def _compute(self):
        self._x_pos = [x / float(self._len - 1) for x in range(self._len)
        ] if self._len != 1 else [.5]  # Center if only one value
        accumulation = [0] * self._len
        for serie in self.series:
            accumulation = map(sum, izip_longest(
                accumulation, [val
                               if val != None else 0
                               for val in serie.values], fillvalue=0))
            serie.points = [
                (self._x_pos[i], v)
                for i, v in enumerate(accumulation)]
            if self.interpolate:
                interpolate = interpolation(
                    self._x_pos, accumulation, kind=self.interpolate)
                p = float(self.interpolation_precision)
                serie.interpolated = [
                    (x / p, float(interpolate(x / p)))
                    for x in range(int(p + 1))
                    if not isnan(float(interpolate(x / p)))]
        return super(StackedLine, self)._compute()



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
import os
from lxml import etree
from pygal.util import template
from pygal import __version__


class Svg(object):
    """Svg object"""
    ns = 'http://www.w3.org/2000/svg'

    def __init__(self, graph):
        self.graph = graph

    def _init(self):
        self.root = etree.Element(
            "{%s}svg" % self.ns,
            nsmap={
                None: self.ns,
                'xlink': 'http://www.w3.org/1999/xlink',
            },
            onload="svg_load();")
        self.root.append(etree.Comment(
            u'Generated with pygal %s ©Kozea 2012' % __version__))
        self.root.append(etree.Comment(u'http://github.com/Kozea/pygal'))
        self.defs = self.node(tag='defs')

    def add_style(self, css):
        style = self.node(self.defs, 'style', type='text/css')
        with open(css) as f:
            templ = template(
                f.read(),
                style=self.graph.style,
                font_sizes=self.graph.font_sizes(),
                hidden='y' if self.graph._horizontal else 'x')
            style.text = templ.decode('utf-8')

    def add_script(self, js):
        script = self.node(self.defs, 'script', type='text/javascript')
        with open(js) as f:
            templ = template(
                f.read(),
                font_sizes=self.graph.font_sizes(False),
                animation_steps=self.graph.animation_steps
            )
            script.text = templ.decode('utf-8')

    def node(self, parent=None, tag='g', attrib=None, **extras):
        if parent is None:
            parent = self.root
        attrib = attrib or {}
        attrib.update(extras)
        for key, value in attrib.items():
            if value is None:
                del attrib[key]
            elif not isinstance(value, basestring):
                attrib[key] = str(value)
            elif key.endswith('_'):
                attrib[key.rstrip('_')] = attrib[key]
                del attrib[key]
        return etree.SubElement(parent, tag, attrib)

    def transposable_node(self, parent=None, tag='g', attrib=None, **extras):
        if self.graph._horizontal:
            for key1, key2 in (('x', 'y'), ('width', 'height')):
                attr1 = extras.get(key1, None)
                attr2 = extras.get(key2, None)
                extras[key1], extras[key2] = attr2, attr1
        return self.node(parent, tag, attrib, **extras)

    def format(self, xy):
        return '%f %f' % xy

    def line(self, node, coords, close=False, **kwargs):
        if len(coords) < 2:
            return
        root = 'M%s L%s Z' if close else 'M%s L%s'
        origin_index = 0
        while None in coords[origin_index]:
            origin_index += 1
        origin = self.format(coords[origin_index])
        line = ' '.join([self.format(c)
                         for c in coords[origin_index + 1:]
                         if None not in c])
        self.node(node, 'path',
                  d=root % (origin, line), **kwargs)

    def _pre_render(self, no_data=False):
        self.add_style(self.graph.base_css or os.path.join(
            os.path.dirname(__file__), 'css', 'graph.css'))
        self.add_script(self.graph.base_js or os.path.join(
            os.path.dirname(__file__), 'js', 'graph.js'))
        self.root.set(
            'viewBox', '0 0 %d %d' % (self.graph.width, self.graph.height))
        if self.graph.explicit_size:
            self.root.set('width', str(self.graph.width))
            self.root.set('height', str(self.graph.height))
        if no_data:
            no_data = self.node(self.root, 'text',
            x=self.graph.width / 2,
            y=self.graph.height / 2,
            class_='no_data')
            no_data.text = self.graph.no_data_text

    def render(self):
        svg = etree.tostring(
            self.root, pretty_print=True,
            xml_declaration=not self.graph.disable_xml_declaration,
            encoding='utf-8')
        if self.graph.disable_xml_declaration:
            svg = svg.decode('utf-8')
        return svg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
import os
import re
from setuptools import setup, find_packages

ROOT = os.path.dirname(__file__)
with open(os.path.join(ROOT, 'pygal', '__init__.py')) as fd:
    __version__ = re.search("__version__ = '([^']+)'", fd.read()).group(1)

setup(
    name="pygal",
    version=__version__,
    description="A python svg graph plotting library",
    author="Kozea",
    url="http://pygal.org/",
    author_email="florian.mounier@kozea.fr",
    license="GNU LGPL v3",
    platforms="Any",
    packages=find_packages(),
    provides=['pygal'],
    scripts=["pygal_gen.py"],
    keywords=[
        "svg", "chart", "graph", "diagram", "plot", "histogram", "kiviat"],
    tests_require=["pytest", "pyquery", "flask", "cairosvg"],
    package_data={'pygal': ['css/*', 'js/*']},
    install_requires=['lxml'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: "
        "GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Topic :: Multimedia :: Graphics :: Presentation"])



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.graph import Graph
from pygal.graph.bar import Bar
from pygal.graph.stackedbar import StackedBar


class HorizontalGraph(Graph):
    """Horizontal graph"""
    def __init__(self, *args, **kwargs):
        self.first_pass = True
        kwargs['_horizontal'] = True
        super(HorizontalGraph, self).__init__(*args, **kwargs)

    def _compute(self):
        self.first_pass = False
        if self.first_pass and self.x_labels:
            self.x_labels = list(reversed(self.x_labels))
        super(HorizontalGraph, self)._compute()
        self._x_labels, self._y_labels = self._y_labels, self._x_labels
        self._box.swap()
        # Y axis is inverted
        if self.first_pass:
            for serie in self.series:
                serie.values = list(reversed(serie.values))


class HorizontalBar(HorizontalGraph, Bar):
    """Horizontal Bar graph"""


class HorizontalStackedBar(HorizontalGraph, StackedBar):
    """Horizontal Stacked Bar graph"""



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.graph.bar import Bar


class StackedBar(Bar):
    """Stacked Bar graph"""

    def _compute(self):
        transposed = zip(*[serie.values for serie in self.series])
        positive_vals = [sum([val
                            if val != None and val > 0 else 0 for val in vals])
                           for vals in transposed]
        negative_vals = [sum([val
                            if val != None and val < 0 else 0 for val in vals])
                           for vals in transposed]

        self._box.ymin, self._box.ymax = (
            min(min(negative_vals), 0), max(max(positive_vals), 0))
        self._length = len(self.series[0].values)

        x_pos = [x / float(self._length)
                 for x in range(self._length + 1)
             ] if self._length > 1 else [0, 1]  # Center if only one value
        y_pos = self._compute_scale(self._box.ymin, self._box.ymax
        ) if not self.y_labels else map(float, self.y_labels)
        self._x_ranges = zip(x_pos, x_pos[1:])

        self._x_labels = self.x_labels and zip(self.x_labels, [
            sum(x_range) / 2 for x_range in self._x_ranges])
        self._y_labels = zip(map(self.format, y_pos), y_pos)

    def _plot(self):
        stack_vals = [[0, 0] for i in range(self._length)]
        for serie in self.series:
            serie_node = self._serie(serie.index)
            stack_vals = self.bar(
                serie_node, serie, [
                tuple((self._x_ranges[i][j], v) for j in range(2))
                for i, v in enumerate(serie.values)],
                stack_vals)



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal.style import DefaultStyle


class FontSizes(object):
    """Container for font sizes"""


class Config(object):
    """Class holding config values"""
    _horizontal = False

    # Graph width
    width = 800
    # Graph height
    height = 600
    # Display values in human readable format (ie: 12.4M)
    human_readable = False
    # Display values in logarithmic scale
    logarithmic = False
    # If set to a filename, this will replace the default css
    base_css = None
    # or default js
    base_js = None
    # Style holding values injected in css
    style = DefaultStyle
    # Various font sizes
    label_font_size = 10
    value_font_size = 8
    tooltip_font_size = 20
    title_font_size = 16
    legend_font_size = 14
    # Specify labels rotation angles in degrees
    x_label_rotation = 0
    y_label_rotation = 0
    # Set to false to remove legend
    show_legend = True
    # Set to false to remove dots
    show_dots = True
    # Size of legend boxes
    legend_box_size = 12
    # X labels, must have same len than data.
    # Leave it to None to disable x labels display.
    x_labels = None
    # You can specify explicit y labels (must be list(int))
    y_labels = None
    # Graph title
    # Leave it to None to disable title.
    title = None
    # Set this to the desired radius in px
    rounded_bars = False
    # Always include x axis
    include_x_axis = False
    # Fill areas under lines
    fill = False
    # Line dots (set it to false to get a scatter plot)
    stroke = True
    # Interpolation, this requires scipy module
    # May be any of ‘linear’, ’nearest’, ‘zero’, ‘slinear’, ‘quadratic, ‘cubic’
    # 'krogh', 'barycentric', 'univariate', or an integer specifying the order
    # of the spline interpolator
    interpolate = None
    # Number of interpolated points between two values
    interpolation_precision = 250
    # Set the ordinate zero value (for filling)
    zero = 0
    # Text to display when no data is given
    no_data_text = "No data"
    # Print values when graph is in non interactive mode
    print_values = True
    # Print zeroes when graph is in non interactive mode
    print_zeroes = False
    # Animate tooltip steps (0 disable animation)
    animation_steps = 0
    # Don't write xml declaration and return unicode instead of string
    # (usefull for writing output in html)
    disable_xml_declaration = False
    # Write width and height attributes
    explicit_size = False

    def __init__(self, **kwargs):
        """Can be instanciated with config kwargs"""
        self.__dict__.update(kwargs)

    def __call__(self, **kwargs):
        """Can be updated with kwargs"""
        self.__dict__.update(kwargs)

    def font_sizes(self, with_unit=True):
        fs = FontSizes()
        for name in dir(self):
            if name.endswith('_font_size'):
                setattr(
                    fs,
                    name.replace('_font_size', ''),
                    ('%dpx' % getattr(self, name)
                    ) if with_unit else getattr(self, name))
        return fs



# -*- coding: utf-8 -*-
# This file is part of pygal
#
# A python svg graph plotting library
# Copyright © 2012 Kozea
#
# This library is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pygal. If not, see <http://www.gnu.org/licenses/>.
from pygal import Line
from pygal.test.utils import texts


def test_logarithmic():
    line = Line(logarithmic=True)
    line.add('_', [1, 10 ** 10, 1])
    q = line.render_pyquery()
    assert len(q(".axis.x")) == 0
    assert len(q(".axis.y")) == 1
    assert len(q(".plot .series path")) == 1
    assert len(q(".legend")) == 1
    assert len(q(".x.axis .guides")) == 0
    assert len(q(".y.axis .guides")) == 51
    assert len(q(".dots")) == 3


def test_logarithmic_bad_interpolation():
    line = Line(logarithmic=True, interpolate='cubic')
    line.add('_', [.001, .00000001, 1])
    q = line.render_pyquery()
    assert len(q(".y.axis .guides")) == 40


def test_logarithmic_big_scale():
    line = Line(logarithmic=True)
    line.add('_', [10 ** -10, 10 ** 10, 1])
    q = line.render_pyquery()
    assert len(q(".y.axis .guides")) == 41


def test_logarithmic_small_scale():
    line = Line(logarithmic=True)
    line.add('_', [1 + 10 ** 10, 3 + 10 ** 10, 2 + 10 ** 10])
    q = line.render_pyquery()
    assert len(q(".y.axis .guides")) == 21


def test_human_readable():
    line = Line()
    line.add('_', [10 ** 4, 10 ** 5, 23 * 10 ** 4])
    # q = line.render_pyquery()
    # assert q(".axis.y text").map(texts) == map(
        # str, range(20000, 240000, 20000))
    line.human_readable = True
    q = line.render_pyquery()
    assert q(".axis.y text").map(texts) == map(
        lambda x: '%dk' % x, range(20, 240, 20))


def test_show_legend():
    line = Line()
    line.add('_', [1, 2, 3])
    q = line.render_pyquery()
    assert len(q(".legend")) == 1
    line.show_legend = False
    q = line.render_pyquery()
    assert len(q(".legend")) == 0


def test_show_dots():
    line = Line()
    line.add('_', [1, 2, 3])
    q = line.render_pyquery()
    assert len(q(".dots")) == 3
    line.show_dots = False
    q = line.render_pyquery()
    assert len(q(".dots")) == 0


def test_no_data():
    line = Line()
    q = line.render_pyquery()
    assert q("text").text() == "No data"
    line.no_data_text = u"þæ®þæ€€&ĳ¿’€"
    q = line.render_pyquery()
    assert q("text").text() == u"þæ®þæ€€&ĳ¿’€"
